package com.mt.wms.empty.task;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.utils.IDGenerator;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;

/**
 * @Author: liguanghao
 * @Date: 2021/11/18 20:27
 * @Version 1.0
 */
@Service
@Transactional
public class AsynRunTaskService extends BaseService {

    @Resource
    private CurrTaskServiceBiz currTaskServiceBiz;
    @Resource
    private RunTaskServiceBiz runTaskServiceBiz;
    @Resource
    private LocationServiceBiz locationServiceBiz;
    @Resource
    private InStockInfoServiceBiz inStockInfoServiceBiz;

    //窑炉可用未满的情况下，调用车辆起点为提升台终点为窑炉
    @Async("asyncServiceExecutor")
    public void asynRunTask(CurrTask currTask){
        //新建一条执行任务的关系表存放任务执行信息
        RunTask runTask=new RunTask();
        runTask.setTaskId(currTask.getId());
        // TODO: 2021/12/13  currTask code、车辆id、开始执行时间

        runTask.setStartPosition(currTask.getStartPosition());
        runTask.setEndPosition(currTask.getTargetPosition());
        setCommonField(runTask);
        runTaskServiceBiz.save(runTask);
        //调用车辆填入起终点

        //得到车辆执行结果


        //执行成功,托盘进炉(存疑，进入窑炉不算任务完成的话，窑炉怎么给出是哪条任务加工完成了)
        if (true){
            //更新关系表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            // TODO: 2021/12/13 结束时间
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表状态
            // TODO: 2021/12/13 is_in 改为 1、runTaskId、起终点
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
        }else {
            // TODO: 2021/12/13  车辆执行不成功，记录日志，推送消息到前端提醒？
        }
    }
    //窑炉已满，调用车辆起点为提升台，终点为缓存区
    @Async("asyncServiceExecutor")
    public void asynRunTaskToWarehouse(CurrTask currTask){
        Location location = locationServiceBiz.list(new QueryWrapper<Location>()
                .eq(Location.STATUS, 1)
                .eq(Location.VALID, 1)).get(0);
        //新建一条执行任务的关系表存放任务执行信息,终点为缓存区空闲库位
        RunTask runTask=new RunTask();
        setCommonField(runTask);
        // TODO: 2021/12/13  currTask code、车辆id、开始执行时间
        runTask.setTaskId(currTask.getId());
        runTask.setStartPosition(currTask.getStartPosition());
        runTask.setEndPosition(location.getCode());
        runTaskServiceBiz.save(runTask);
        //更新 location 表对应库位状态为占用
        location.setStatus(1);
        location.setUpdateTime(LocalDateTime.now());
        locationServiceBiz.updateById(location);
        //添加一条库位详情到in_stock_info表
        InStockInfo inStockInfo=new InStockInfo();
        inStockInfo.setStatus(1);
        inStockInfo.setInterCode(IDGenerator.gen("HCT", "yyyyMMddHHmm", 2, "WAREHOUSE_CODE"));
        inStockInfo.setTaskId(currTask.getId());
        inStockInfo.setTaskCode(currTask.getTaskCode());
        inStockInfo.setKilnId(currTask.getKilnId());
        inStockInfo.setKilnName(currTask.getKilnName());
        inStockInfo.setLocationId(location.getId());
        inStockInfo.setLocationName(location.getLocationNameAlias());
        setCommonField(inStockInfo);
        inStockInfoServiceBiz.save(inStockInfo);
        //修改currTask的是否缓存,库位id,库位名称
        currTask.setIsCache(1);
        currTask.setLocationId(location.getId());
        currTask.setLocationName(location.getLocationNameAlias());
        currTaskServiceBiz.updateById(currTask);
        //调用车辆

        if (true){
            //更新关系表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            // TODO: 2021/12/13 结束时间
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表
            // TODO: 2021/12/13 is_cache 、runTaskId
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
        }else {
            // TODO: 2021/12/13  车辆执行不成功，记录日志，推送消息到前端提醒？
        }

        // TODO: 2021/12/13 车辆执行执行成功后加入窑炉的缓存区任务队列

    }
}
